import logging
from .model.agent_card import AgentCard

class AgentResolver:
    def __init__(self):
        pass

    # find the best agent and tool_name
    async def resolve(self, arguments: dict = None, agents: list[AgentCard] = None) -> tuple[AgentCard, str]:
        if list is None or len(list) == 0:
            logging.error('The agents list is empty!')
            return None
        if arguments is None or len(arguments):
            logging.warning('The arguments is empty!')
        agent = agents[0]
        tool_name = None
        return (agent, tool_name)



import asyncio
import os
from mcp import ClientSession
from mcp.client.streamable_http import streamablehttp_client

# Replace with your actual MCP server URL
MCP_SERVER_URL = "http://localhost:8000/mcp"

async def main():
    async with streamablehttp_client(MCP_SERVER_URL) as (read, write, _):
        async with ClientSession(read, write) as session:
            await session.initialize()
            tools = await session.list_tools()
            print("Available tools:")
            for tool in tools.tools:
                print(f"- {tool.name}: {tool.description}")
            
            # Call the 'add' tool
            result = await session.call_tool("add", {"a": 2, "b": 3})
            print(f"add(2, 3) result: {result.structuredContent if hasattr(result, 'structuredContent') else result}")

if __name__ == "__main__":
    asyncio.run(main())



import asyncio
import os
from mcp import ClientSession
from mcp.client.streamable_http import streamablehttp_client
from ..model.agent_card import AgentCard

class MCPAgentCaller:
    def __init__(self, agent_card: AgentCard):
        self.agent_card = agent_card

    async def call(self, tool_name: str, arguments: dict = None) -> dict:
        async with streamablehttp_client(self.agent_card.url) as (read, write, _):
            async with ClientSession(read, write) as session:
                await session.initialize()    

                result = await session.call_tool(tool_name, arguments)
                print(f"{tool_name, arguments}, result: {result.structuredContent if hasattr(result, 'structuredContent') else result}")
                return result


from typing_extensions import override
from a2a.server.apps import A2AStarletteApplication
from a2a.server.request_handlers import DefaultRequestHandler
from a2a.server.tasks import InMemoryTaskStore
from a2a.server.agent_execution import AgentExecutor, RequestContext
from a2a.server.events import EventQueue
from a2a.types import AgentCard, AgentSkill, AgentCapabilities
from a2a.utils import new_agent_text_message
import uvicorn

# Define the Agent Skill
skill = AgentSkill(
    id='hello_world',
    name='Returns hello world',
    description='Just returns hello world',
    tags=['hello world'],
    examples=['hi', 'hello world'],
)

# Define the Agent Card (simplified, without AgentAuthentication)
agent_card = AgentCard(
    name='Hello World Agent',
    description='Just a hello world agent',
    url='http://localhost:9999/',
    version='1.0.0',
    defaultInputModes=['text'],
    defaultOutputModes=['text'],
    capabilities=AgentCapabilities(),
    skills=[skill],
    authentication=['public'],  # Use a list of schemes instead
)

# Define the Agent Logic
class HelloWorldAgent:
    """Hello World Agent."""
    async def invoke(self) -> str:
        return 'Hello World'

# Define the Agent Executor
class HelloWorldAgentExecutor(AgentExecutor):
    """Agent Executor for Hello World Agent."""
    def __init__(self):
        self.agent = HelloWorldAgent()

    @override
    async def execute(self, context: RequestContext, event_queue: EventQueue) -> None:
        result = await self.agent.invoke()
        event_queue.enqueue_event(new_agent_text_message(result))

    @override
    async def cancel(self, context: RequestContext, event_queue: EventQueue) -> None:
        raise Exception('cancel not supported')

# Main Server Setup
if __name__ == '__main__':
    # Set up the request handler
    request_handler = DefaultRequestHandler(
        agent_executor=HelloWorldAgentExecutor(),
        task_store=InMemoryTaskStore(),
    )

    # Create the A2A Starlette application
    server_app_builder = A2AStarletteApplication(
        agent_card=agent_card,
        http_handler=request_handler
    )

    # Start the server
    uvicorn.run(server_app_builder.build(), host='0.0.0.0', port=9999)


import httpx
from a2a.client import A2AClient
from a2a.types import AgentCard as A2AAgentCard
from ..model.agent_card import AgentCard

class A2AAgentCaller:
    def __init__(self, agent_card: A2AAgentCard):
        self.agent_card = agent_card
    
    async def call(self, tool_name: str, arguments: dict = None) -> dict:
        """Call a specific tool on the agent with given arguments."""
        async with httpx.AsyncClient() as http_client:
            try:
                # Initialize A2AClient
                client = A2AClient(self.agent_card.url, http_client=http_client)
                
                # Verify tool exists in AgentCard
                skill_ids = [skill.id for skill in self.agent_card.skills]
                if tool_name not in skill_ids:
                    raise ValueError(f"Tool '{tool_name}' not found. Available tools: {skill_ids} for agent: {self.agent_card.name}")
                
                # Call the tool with arguments (default to empty dict if None)
                response = await client.call_tool(tool_name=tool_name, arguments=arguments or {})
                return response
            except Exception as e:
                raise Exception(f"Error calling tool '{tool_name} for agent: {self.agent_card.name}': {e}")


from mcp.server.fastmcp import FastMCP

# Create a FastMCP server instance
mcp = FastMCP("Quickstart Server")

# Add a simple tool
@mcp.tool()
def add(a: int, b: int) -> int:
    """Add two numbers"""
    return a + b

@mcp.tool()
def hello(name: str = "World") -> str:
    """Say hello to someone."""
    return f"Hello, {name}!"

if __name__ == "__main__":
    mcp.run(transport="streamable-http")



from typing import List, Optional
from pydantic import BaseModel


class Skill(BaseModel):
    description: str


class Provider(BaseModel):
    name: str
    url: str


from typing import ClassVar

class AgentCard(BaseModel):
    TYPE_A2A: ClassVar[str] = 'type_a2a'
    TYPE_MCP: ClassVar[str] = 'mcp_type'
    url: str
    name: str
    type: Optional[str] = None
    tag: Optional[str] = None
    github: Optional[str] = None
    skills: Optional[List[Skill]] = None
    status: Optional[str] = None
    version: Optional[str] = None
    examples: Optional[List[str]] = None
    provider: Optional[Provider] = None
    description: Optional[str] = None
    lastRelease: Optional[str] = None
    availability: Optional[str] = None
    monthlyCalls: Optional[str] = None
    documentation: Optional[str] = None



import requests
import os
from dotenv import load_dotenv
from typing import Optional, List
from .model.agent_card import AgentCard
from .agent import Agent
from typing import Any


class AgentNameServiceClient:
    DEFAULT_SERVICE_URL = "https://app.clearentitlement.com"
    PATH_GET_AGENTS = "/ce/admin/Agents"
    agents: dict[str, Agent]

    def __init__(self, service_url: Optional[str] = None):
        self.service_url = (service_url or self.DEFAULT_SERVICE_URL).rstrip("/")
        self.agents = dict()  # Initialize agents dict

    def get_agent_card(self, name: str) -> AgentCard:
        response = requests.get(f"{self.service_url}/agents/{name}")
        response.raise_for_status()
        return AgentCard(**response.json())

    def list_agents(self) -> List[AgentCard]:
        response = requests.get(f"{self.service_url}/agents")
        response.raise_for_status()
        return [AgentCard(**agent) for agent in response.json()]

    def lookup(self, name: Optional[str] = None) -> dict[str, Agent] | Agent | None:
        if not self.agents:
            self.agents = self._get_agents()
        if name is None:
            return self.agents
        if not self.agents.get(name):
            return None
        # If name is provided, get the specific agent card, otherwise use the first one
        return self.agents.get(name)
    
    
    def _get_agents(self) -> dict[str, Agent]:
        """
        Fetch agents from the API and return a dict mapping agent name to Agent instance.
        """
        print("Fetching agents from API...")
        url = f"{self.service_url}{self.PATH_GET_AGENTS}"
        api_key = os.getenv("AGNS_API_KEY")
        if not api_key:
            raise ValueError("AGNS_API_KEY environment variable is not set")
        print("API Key:", api_key)  # Debugging line to check API key

        headers = {"ApiKey": api_key}
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        agents_json = response.json()
        print("API response:", agents_json)

        agents_dict = dict[str, Agent]()
        for agent in agents_json:
            try:
                card = self._convert_agent_api_response(agent)
                agents_dict[card.name] = Agent(card=card)
            except Exception as e:
                print(f"Skipping agent due to error: {e}, data: {agent}")

        print("Total agents fetched:", len(agents_dict))
        return agents_dict
            
    def _convert_agent_api_response(self, agent: dict[str, Any]) -> AgentCard:
        """
        Convert API agent dict to AgentCard-compatible object.
        Adjust the mapping as needed based on your API response.
        """
        metadata = agent.get("metadata", {})
        return AgentCard(**metadata)

class LazyClient:
    def __init__(self):
        self._client: Optional[AgentNameServiceClient] = None

    def _get_default_client(self) -> AgentNameServiceClient:
        load_dotenv()  # Load .env once, only when needed
        return AgentNameServiceClient()

    def __getattr__(self, item: str) -> Any:
        if self._client is None:
            self._client = self._get_default_client()
        return getattr(self._client, item)


# src/aans_sdk/agent.py
from typing import Any, Dict
import requests
from .model.agent_card import AgentCard
from .a2a.a2a_agent_caller import A2AAgentCaller
from .mcp.mcp_agent_caller import MCPAgentCaller
from .agent_resolver import AgentResolver

# from pydantic_ai import Agent as PydanticAgent

class Agent:
    def __init__(self, card: AgentCard):
        # self.agent = PydanticAgent(**card.model_dump())
        self.card = card

    def ask(self, question: Dict[str, Any]) -> Dict[str, Any]:
        print(f"Asking agent {self.card.name} with type of {self.card.type} with question: {question}")
        agent_resolve = AgentResolver()
        (agent_card, tool_name) = agent_resolve.resolve(question, [self.card])
        if (self.card.type == AgentCard.TYPE_A2A):
            a2a_agent = A2AAgentCaller(agent_card)
            return a2a_agent.call(tool_name, question)
        if (self.card.type == AgentCard.TYPE_MCP):
            mcp_agent = MCPAgentCaller(agent_card)
            return mcp_agent.call(tool_name, question)
        response = requests.post(self.card.url, json=question)
        response.raise_for_status()
        return response.json()
    
    def invoke(self, verb: str, question: Dict[str, Any]) -> Dict[str, Any]:
        print(f"Performing verb '{verb}' on agent {self.card.name} with question: {question}")
        # Assuming the agent's URL can handle different verbs as part of the path or query
        # This is a placeholder; you might need to adjust the URL or request body
        # based on how your agent API is designed to handle different verbs.
        tool_name = verb # assume verb is the tool_name
        if (self.card.type == AgentCard.TYPE_A2A):
            a2a_agent = A2AAgentCaller(self.card)
            return a2a_agent.call(tool_name, question)
        if (self.card.type == AgentCard.TYPE_MCP):
            mcp_agent = MCPAgentCaller(self.card)
            return mcp_agent.call(tool_name, question)
        response = requests.post(f"{self.card.url}/{verb}", json=question)
        response.raise_for_status()
        return response.json()

    def describe(self) -> Dict[str, Any]:
        return self.card.model_dump()
